import dotenv from 'dotenv';
import { FisAppActor, FisMessage, TransmissionMessage } from "../interface/transport.interface";
import { ConnectionAdapter } from "./connector.base";
import { AdaptorTransmissionRole, ConnectionState, TransmitterConnectionAdapter as TransmitterConnectionAdapterInterface, Transport, TransportMessage, TransportService } from '../interface/connector.interface';
import { BehaviorSubject, Observable, Observer } from 'rxjs';
import { v4 as uuidv4 } from 'uuid'
import { WebsocketTransportService } from '../transport/websocket';

dotenv.config();
/* This transport manager will be instantiating the necessary transport to deal with tranmission and receiving from different receivers
So how?: */
export class TransmitterConnectionAdapter extends ConnectionAdapter implements TransmitterConnectionAdapterInterface {
    connectionStateBus: BehaviorSubject<ConnectionState> = new BehaviorSubject('OFFLINE' as ConnectionState)

    constructor(transportService: TransportService) {
        super()
        // logic here
        this.connector = transportService
    }

    emitStream(message: TransportMessage): void {
        throw new Error('Method not implemented.');
    }

    emit(message: TransmissionMessage): void {
        (this.connector as TransportService).emit({
            id: uuidv4(),
            dateCreated: new Date(),
            transport: Transport.Websocket,
            target: message.receiver?.id,
            payload: message
        } as TransportMessage)
    }
}


