import { MessageTransmissionTransmitter } from "./msg.transmission.transmitter";
import { MessageTransmissionReceiver } from "./msg.transmission.receiver";
import { ConnectionManager } from "../connector/connector.manager";
import { MessageTransmission, MessageTransmissionManager as MessageTransmissionManagerInterface, ReceiverProfile, TransmissionEvent, TransmissionProfile, TransmitterProfile } from "../interface/transport.interface";
import { v4 as uuidv4 } from 'uuid'
import { AdaptorTransmissionRole, RequestResponseConnectionAdapter } from "../interface/connector.interface";
import { MessageTransmissionRequestResponse } from "./msg.transmission.request-response";
import { TransmitterConnectionAdapter } from "../connector/connector.transmitter";
import { ReceiverConnectionAdapter } from "../connector/connector.receiver";
import { filter, Subject } from "rxjs";
/* FOr now, to fill the holes/gaps so to speak, the FIS actor will interact wtih Message Transmission Manager, without having to wory about
the underyling transport protocol as well as managing client control, when it's acting as a transmitter. */
export class MessageTransmissionManager implements MessageTransmissionManagerInterface {
    // transmission: MessageTransmission[] = []
    connectionManager!: ConnectionManager
    messageTransmissionTransmitters: MessageTransmissionTransmitter[] = []
    messageTransmissionReceiver: MessageTransmissionReceiver[] = []
    messageTransmissionRequestResponse: MessageTransmissionRequestResponse[] = []
    transmissionEvent: Subject<TransmissionEvent> = new Subject()

    constructor() {
        // this.transmissionEvent.pipe(
        //     filter(event => event.event == 'NewTransmissionSet') // filter out odd numbers
        // ).subscribe((transmissionData: TransmissionEvent) => {
        //     this.transmission.push(transmissionData.data as MessageTransmission)
        // })
    }

    // but this function also needs to talk to connection adaptor manager to instantiate the necessary adapter.
    getTransmissionInstance(): MessageTransmission {
        let transmitterAdapter: TransmitterConnectionAdapter = this.connectionManager.getTransmitterConnectionAdapter()
        let receiverAdapter: ReceiverConnectionAdapter = this.connectionManager.getReceiverConnectionAdapter()
        let transmitter: MessageTransmissionTransmitter = this.getTransmitter(transmitterAdapter)
        let receiver: MessageTransmissionReceiver = this.getReceiver(receiverAdapter)
        let transmission: MessageTransmission = {
            id: uuidv4(),
            receiverId: transmitter.getInfo().id,
            transmitterId: receiver.getInfo().id,
            transmitter: transmitter,
            receiver: receiver,
            requestResponse: this.getRequestResponse(transmitter, receiver, this.connectionManager.getRequestResponseConnectionAdapter(transmitterAdapter, receiverAdapter))
        }
        this.instantiateConnectionManager() // start an adapter
        // this.transmission.push(transmission)
        return transmission
    }

    private getTransmitter(connectionAdapter: TransmitterConnectionAdapter): MessageTransmissionTransmitter {
        let transmitterProfile: TransmitterProfile = {
            id: uuidv4(),
            name: '', // for now make it empty. We will use the assigned uuid here
            dateCreated: new Date()
        }
        return new MessageTransmissionTransmitter(transmitterProfile, AdaptorTransmissionRole.Transmitter, connectionAdapter)
    }

    private getReceiver(connectionAdapter: ReceiverConnectionAdapter): MessageTransmissionReceiver {
        let receiverProfile: ReceiverProfile = {
            id: uuidv4(),
            name: '', // for now make it empty. We will use the assigned uuid here
            dateCreated: new Date()
        }
        return new MessageTransmissionReceiver(receiverProfile, AdaptorTransmissionRole.Receiver, connectionAdapter)
    }

    private getRequestResponse(transmitterInstance: MessageTransmissionTransmitter, receiverInstance: MessageTransmissionReceiver, connectionAdaptor: RequestResponseConnectionAdapter): MessageTransmissionRequestResponse {
        return new MessageTransmissionRequestResponse(transmitterInstance, receiverInstance, AdaptorTransmissionRole.RequestResponse, connectionAdaptor)
    }

    private instantiateConnectionManager(): void {
        this.connectionManager = new ConnectionManager(this.transmissionEvent)
    }

}