import { filter, Observable, Subject } from "rxjs";
import { Socket as ClientSocket } from 'socket.io-client'
import { Socket as SocketForConnectedClient } from "socket.io"
import { handleClientSocketConnection, handleNewSocketClient, startClientSocketConnection, startSocketServer } from "../utils/socket.utils";
import { TransportEvent, TransportMessage, TransportService } from "../interface/connector.interface";

/* Just code in the context that this websocket service will be handling multiple UI clients. Can think about the server communication at a later time. */
export class WebsocketTransportService implements TransportService {
    private connectedServer: ConnectedServerSocket[] = []
    private connectedClientSocket: ConnectedClientSocket[] = []
    // private incomingMessage: Subject<TransportMessage> = new Subject() // this is only for client roles only atm
    private transportEvent: Subject<TransportEvent> = new Subject()

    constructor() {
        // logic here
    }

    public startServer(port: number): void {
        // logic here
        startSocketServer(port).subscribe({
            next: (connectedClient: SocketForConnectedClient) => {
                handleNewSocketClient(connectedClient, this.connectedClientSocket).subscribe(this.transportEvent)
            },
            error: error => console.error(error),
        })
    }

    public startClient(url: string): void {
        // logic here
        startClientSocketConnection(url).then((socket: ClientSocket) => {
            handleClientSocketConnection(socket, this.connectedServer).subscribe(this.transportEvent)
        }).catch((error) => {
            console.error(`WebsocketTransport ERROR:`, error)
        })
    }

    public getTransportEvent(): Observable<TransportEvent> {
        return this.transportEvent.asObservable()
    }

    
    // for transmission(Server Only, not applicable for client Socket)
    public emit(message: TransportMessage): void {
        // send message
    }
    
    public subscribe(): Observable<TransportEvent> {
        return this.transportEvent.asObservable()
    }


}



interface ConnectedSocket {
    id: string,
    dateCreated: Date,
}
export interface ConnectedClientSocket extends ConnectedSocket {
    socketInstance: SocketForConnectedClient
}

export interface ConnectedServerSocket extends ConnectedSocket {
    socketInstance: ClientSocket
}